@@ -69,6 +69,8 @@ Any comments/updates/bug reports to:
 #include <stdarg.h>
 #include <stdlib.h>
 #include "paging.h"
+#include "logging.h"
+#include "cpu.h"
 
 typedef uint8_t  UINT8;
 typedef uint16_t UINT16;
@@ -103,6 +105,8 @@ INLINE UINT32 le_uint32(const void* ptr) {
 static UINT8 must_do_size;   /* used with size of operand */
 static int wordop;           /* dealing with word or byte operand */
 
+static uint8_t last_c = 0;
+
 static uint32_t instruction_offset;
 //static UINT16 instruction_segment;
 
@@ -110,6 +114,7 @@ static char* ubufs;           /* start of buffer */
 static char* ubufp;           /* last position of buffer */
 static int invalid_opcode = 0;
 static int first_space = 1;
+static uint8_t last_prefix = MP_NONE;
 
 static int prefix;            /* segment override prefix byte */
 static int modrmv;            /* flag for getting modrm byte */
@@ -275,15 +280,15 @@ static char const *second[] = {
   "invd",             "wbinvd",          0,                "UD2",
   0,                  0,                 0,                0,
 /* 1 */
-  "mov %Eb,%Gb",      "mov %Ev,%Gv",     "mov %Gb,%Eb",    "mov %Gv,%Ev",
-  0,                  0,                 0,                0,
-  0,                  0,                 0,                0,
+  "%x0",              "%x0",             "%x0",            "%x0",
+  "%x0",              "%x0",             "%x0",            "%x0",
+  "%g=",              0,                 0,                0,
   0,                  0,                 0,                0,
 /* 2 */
   "mov %Rd,%Cd",      "mov %Rd,%Dd",     "mov %Cd,%Rd",    "mov %Dd,%Rd",
   "mov %Rd,%Td",      0,                 "mov %Td,%Rd",    0,
-  0,                  0,                 0,                0,
-  0,                  0,                 0,                0,
+  "%x0",              "%x0",             "%x0",            "%x0",
+  "%x0",              "%x0",             "%x0",            "%x0",
 /* 3 */
   0,                  "rdtsc",           0,                0,
   "sysenter",         "sysexit",         0,                0,
@@ -295,15 +300,15 @@ static char const *second[] = {
   "cmovs %Gv,%Ev",    "cmovns %Gv,%Ev",  "cmovp %Gv,%Ev",  "cmovnp %Gv,%Ev",
   "cmovl %Gv,%Ev",    "cmovge %Gv,%Ev",  "cmovle %Gv,%Ev", "cmovg %Gv,%Ev",
 /* 5 */
-  0, 0, 0, 0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0, 0, 0, 0,
+  "%x0", "%x0", "%x0", "%x0", "%x0", "%x0", "%x0", "%x0",
+  "%x0", "%x0", 0,     0,     "%x0", "%x0", "%x0", "%x0",
 /* 6 */
   "punpcklbw %GM,%EM","punpcklwd %GM,%EM","punpckldq %GM,%EM","packsswb %GM,%EM",
   "pcmpgtb %GM,%EM",  "pcmpgtw %GM,%EM", "pcmpgtd %GM,%EM","packuswb %GM,%EM",
   "punpckhbw %GM,%EM","punpckhwd %GM,%EM","punpckhdq %GM,%EM","packssdw %GM,%EM",
   0,                  0,                 "movd %GM,%Ed",   "movq %GM,%EM",
 /* 7 */
-  0,                  "%g;",             "%g:",            "%g9",
+  "pshufw %GM,%EM,%Ib","%g;",            "%g:",            "%g9",
   "pcmpeqb %GM,%EM",  "pcmpeqw %GM,%EM", "pcmpeqd %GM,%EM","emms",
   0,                  0,                 0,                0,
   0,                  0,                 "movd %Ed,%GM",   "movq %EM,%GM",
@@ -321,34 +326,176 @@ static char const *second[] = {
   "push fs",          "pop fs",          "cpuid",          "bt %Ev,%Gv",
   "shld %Ev,%Gv,%Ib", "shld %Ev,%Gv,cl", 0,                0,
   "push gs",          "pop gs",          0,                "bts %Ev,%Gv",
-  "shrd %Ev,%Gv,%Ib", "shrd %Ev,%Gv,cl", 0,                "imul %Gv,%Ev",
+  "shrd %Ev,%Gv,%Ib", "shrd %Ev,%Gv,cl", "%g<",            "imul %Gv,%Ev",
 /* b */
   "cmpxchg %Eb,%Gb",  "cmpxchg %Ev,%Gv", "lss %Mp",        "btr %Ev,%Gv",
   "lfs %Mp",          "lgs %Mp",         "movzx %Gv,%Eb",  "movzx %Gv,%Ew",
   0,                  0,                 "%g7 %Ev,%Ib",    "btc %Ev,%Gv",
   "bsf %Gv,%Ev",      "bsr %Gv,%Ev",     "movsx %Gv,%Eb",  "movsx %Gv,%Ew",
 /* c */
-  "xadd %Eb,%Gb",     "xadd %Ev,%Gv",    0,                0,
-  0,                  0,                 0,                "%g8",
+  "xadd %Eb,%Gb",     "xadd %Ev,%Gv",    "%x0",            0,
+  "%x0",              "%x0",             "%x0",            "%g8",
   "bswap eax",        "bswap ecx",       "bswap edx",      "bswap ebx",
   "bswap esp",        "bswap ebp",       "bswap esi",      "bswap edi",
 /* d */
   0,                  "psrlw %GM,%EM",   "psrld %GM,%EM",  "psrlq %GM,%EM",
-  "paddq %GM,%EM",    "pmullw %GM,%EM",  0,                0,
-  "psubusb %GM,%EM",  "psubusw %GM,%EM", 0,                "pand %GM,%EM",
-  "paddusb %GM,%EM",  "paddusw %GM,%EM", 0,                "pandn %GM,%EM",
+  "paddq %GM,%EM",    "pmullw %GM,%EM",  0,                "%x0",
+  "psubusb %GM,%EM",  "psubusw %GM,%EM", "%x0",            "%x0",
+  "paddusb %GM,%EM",  "paddusw %GM,%EM", "%x0",            "pandn %GM,%EM",
 /* e */
-  0,                  "psraw %GM,%EM",   "psrad %GM,%EM",  0,
-  0,                  "pmulhw %GM,%EM",  0,                0,
-  "psubsb %GM,%EM",   "psubsw %GM,%EM",  0,                "por %GM,%EM",
-  "paddsb %GM,%EM",   "paddsw %GM,%EM",  0,                "pxor %GM,%EM",
+  "%x0",              "psraw %GM,%EM",   "psrad %GM,%EM",  "%x0",
+  "%x0",              "%x0",             0,                "%x0",
+  "%x0",              "%x0",             "%x0",            "%x0",
+  "%x0",              "%x0",             "%x0",            "%x0",
 /* f */
   0,                  "psllw %GM,%EM",   "pslld %GM,%EM",  "psllq %GM,%EM",
-  0,                  "pmaddwd %GM,%EM", 0,                0,
+  0,                  "pmaddwd %GM,%EM", "%x0",            "%x0",
   "psubb %GM,%EM",    "psubw %GM,%EM",   "psubd %GM,%EM",  0,
   "paddb %GM,%EM",    "paddw %GM,%EM",   "paddd %GM,%EM",  0
 };
 
+static char const *mpgroups[][256][4] = { /* mandatory prefix groups SSE instructions, then within NP, 66, F2, F3 */
+/* group 0 */
+  {
+    /* 0x00 */ { 0,0,0,0 }, /* 0x01 */ { 0,0,0,0 }, /* 0x02 */ { 0,0,0,0 }, /* 0x03 */ { 0,0,0,0 },
+    /* 0x04 */ { 0,0,0,0 }, /* 0x05 */ { 0,0,0,0 }, /* 0x06 */ { 0,0,0,0 }, /* 0x07 */ { 0,0,0,0 },
+    /* 0x08 */ { 0,0,0,0 }, /* 0x09 */ { 0,0,0,0 }, /* 0x0A */ { 0,0,0,0 }, /* 0x0B */ { 0,0,0,0 },
+    /* 0x0C */ { 0,0,0,0 }, /* 0x0D */ { 0,0,0,0 }, /* 0x0E */ { 0,0,0,0 }, /* 0x0F */ { 0,0,0,0 },
+
+    /* 0x10 */ { "movups %GX,%EX", "movupd %GX,%EX", "movsd %GX,%EX", "movss %GX,%EX" },
+    /* 0x11 */ { "movups %EX,%GX", "movupd %EX,%GX", "movsd %EX,%GX", "movss %EX,%GX" },
+    /* 0x12 */ { "movlps %GX,%EX", 0,0,0 }, // FIXME: The design of this disassembler does not allow changing opcode name based on r/m = memory or r/m = reg
+    /* 0x13 */ { "movlps %EX,%GX", 0,0,0 },
+    /* 0x14 */ { "unpcklps %GX,%EX", 0,0,0 },
+    /* 0x15 */ { "unpckhps %GX,%EX", 0,0,0 },
+    /* 0x16 */ { "movhps %GX,%EX", 0,0,0 }, // FIXME: The design of this disassembler does not allow changing opcode name based on r/m = memory or r/m = reg
+    /* 0x17 */ { "movhps %EX,%GX", 0,0,0 },
+    /* 0x18 */ { 0,0,0,0 }, /* 0x19 */ { 0,0,0,0 }, /* 0x1A */ { 0,0,0,0 }, /* 0x1B */ { 0,0,0,0 },
+    /* 0x1C */ { 0,0,0,0 }, /* 0x1D */ { 0,0,0,0 }, /* 0x1E */ { 0,0,0,0 }, /* 0x1F */ { 0,0,0,0 },
+
+    /* 0x20 */ { 0,0,0,0 }, /* 0x21 */ { 0,0,0,0 }, /* 0x22 */ { 0,0,0,0 }, /* 0x23 */ { 0,0,0,0 },
+    /* 0x24 */ { 0,0,0,0 }, /* 0x25 */ { 0,0,0,0 }, /* 0x26 */ { 0,0,0,0 }, /* 0x27 */ { 0,0,0,0 },
+    /* 0x28 */ { "movaps %GX,%EX", "movapd %GX,%EX", 0, 0 },
+    /* 0x29 */ { "movaps %EX,%GX", "movapd %EX,%GX", 0, 0 },
+    /* 0x2A */ { "cvtpi2ps %GX,%EM", 0, 0, "cvtsi2ss %GX,%Ed" },
+    /* 0x2B */ { "movntps %EX,%GX", 0,0,0 },
+    /* 0x2C */ { "cvttps2pi %GM,%EX", 0, 0, "cvttss2si %Gd,%EX" },
+    /* 0x2D */ { "cvtps2pi %GM,%EX", 0, 0, "cvtss2si %Gd,%EX" },
+    /* 0x2E */ { "ucomiss %GX,%EX", 0,0,0 },
+    /* 0x2F */ { "comiss %GX,%EX", 0,0,0 },
+
+    /* 0x30 */ { 0,0,0,0 }, /* 0x31 */ { 0,0,0,0 }, /* 0x32 */ { 0,0,0,0 }, /* 0x33 */ { 0,0,0,0 },
+    /* 0x34 */ { 0,0,0,0 }, /* 0x35 */ { 0,0,0,0 }, /* 0x36 */ { 0,0,0,0 }, /* 0x37 */ { 0,0,0,0 },
+    /* 0x38 */ { 0,0,0,0 }, /* 0x39 */ { 0,0,0,0 }, /* 0x3A */ { 0,0,0,0 }, /* 0x3B */ { 0,0,0,0 },
+    /* 0x3C */ { 0,0,0,0 }, /* 0x3D */ { 0,0,0,0 }, /* 0x3E */ { 0,0,0,0 }, /* 0x3F */ { 0,0,0,0 },
+
+    /* 0x40 */ { 0,0,0,0 }, /* 0x41 */ { 0,0,0,0 }, /* 0x42 */ { 0,0,0,0 }, /* 0x43 */ { 0,0,0,0 },
+    /* 0x44 */ { 0,0,0,0 }, /* 0x45 */ { 0,0,0,0 }, /* 0x46 */ { 0,0,0,0 }, /* 0x47 */ { 0,0,0,0 },
+    /* 0x48 */ { 0,0,0,0 }, /* 0x49 */ { 0,0,0,0 }, /* 0x4A */ { 0,0,0,0 }, /* 0x4B */ { 0,0,0,0 },
+    /* 0x4C */ { 0,0,0,0 }, /* 0x4D */ { 0,0,0,0 }, /* 0x4E */ { 0,0,0,0 }, /* 0x4F */ { 0,0,0,0 },
+
+    /* 0x50 */ { "movmskps %Gd,%EX", 0,0,0 },
+    /* 0x51 */ { "sqrtps %GX,%EX", "sqrtpd %GX,%EX", "sqrtsd %GX,%EX", "sqrtss %GX,%EX" },
+    /* 0x52 */ { "rqsrtps %GX,%EX", 0,0, "rsqrtss %GX,%EX" },
+    /* 0x53 */ { "rcpps %GX,%EX", 0,0, "rcpss %GX,%EX" },
+    /* 0x54 */ { "andps %GX,%EX", "andpd %GX,%EX", 0,0 },
+    /* 0x55 */ { "andnps %GX,%EX", "andnpd %GX,%EX", 0,0 },
+    /* 0x56 */ { "orps %GX,%EX", "orpd %GX,%EX", 0,0 },
+    /* 0x57 */ { "xorps %GX,%EX", "xorpd %GX,%EX", 0,0 },
+    /* 0x58 */ { "addps %GX,%EX", "addpd %GX,%EX", "addsd %GX,%EX", "addss %GX,%EX" },
+    /* 0x59 */ { "mulps %GX,%EX", "mulpd %GX,%EX", "mulsd %GX,%EX", "mulss %GX,%EX" },
+    /* 0x5A */ { 0,0,0,0 },
+    /* 0x5B */ { 0,0,0,0 },
+    /* 0x5C */ { "subps %GX,%EX", "subpd %GX,%EX", "subsd %GX,%EX", "subss %GX,%EX" },
+    /* 0x5D */ { "minps %GX,%EX", "minpd %GX,%EX", "minsd %GX,%EX", "minss %GX,%EX" },
+    /* 0x5E */ { "divps %GX,%EX", "divpd %GX,%EX", "divsd %GX,%EX", "divss %GX,%EX" },
+    /* 0x5F */ { "maxps %GX,%EX", "maxpd %GX,%EX", "maxsd %GX,%EX", "maxss %GX,%EX" },
+
+    /* 0x60 */ { 0,0,0,0 }, /* 0x61 */ { 0,0,0,0 }, /* 0x62 */ { 0,0,0,0 }, /* 0x63 */ { 0,0,0,0 },
+    /* 0x64 */ { 0,0,0,0 }, /* 0x65 */ { 0,0,0,0 }, /* 0x66 */ { 0,0,0,0 }, /* 0x67 */ { 0,0,0,0 },
+    /* 0x68 */ { 0,0,0,0 }, /* 0x69 */ { 0,0,0,0 }, /* 0x6A */ { 0,0,0,0 }, /* 0x6B */ { 0,0,0,0 },
+    /* 0x6C */ { 0,0,0,0 }, /* 0x6D */ { 0,0,0,0 }, /* 0x6E */ { 0,0,0,0 }, /* 0x6F */ { 0,0,0,0 },
+
+    /* 0x70 */ { 0,0,0,0 }, /* 0x71 */ { 0,0,0,0 }, /* 0x72 */ { 0,0,0,0 }, /* 0x73 */ { 0,0,0,0 },
+    /* 0x74 */ { 0,0,0,0 }, /* 0x75 */ { 0,0,0,0 }, /* 0x76 */ { 0,0,0,0 }, /* 0x77 */ { 0,0,0,0 },
+    /* 0x78 */ { 0,0,0,0 }, /* 0x79 */ { 0,0,0,0 }, /* 0x7A */ { 0,0,0,0 }, /* 0x7B */ { 0,0,0,0 },
+    /* 0x7C */ { 0,0,0,0 }, /* 0x7D */ { 0,0,0,0 }, /* 0x7E */ { 0,0,0,0 }, /* 0x7F */ { 0,0,0,0 },
+
+    /* 0x80 */ { 0,0,0,0 }, /* 0x81 */ { 0,0,0,0 }, /* 0x82 */ { 0,0,0,0 }, /* 0x83 */ { 0,0,0,0 },
+    /* 0x84 */ { 0,0,0,0 }, /* 0x85 */ { 0,0,0,0 }, /* 0x86 */ { 0,0,0,0 }, /* 0x87 */ { 0,0,0,0 },
+    /* 0x88 */ { 0,0,0,0 }, /* 0x89 */ { 0,0,0,0 }, /* 0x8A */ { 0,0,0,0 }, /* 0x8B */ { 0,0,0,0 },
+    /* 0x8C */ { 0,0,0,0 }, /* 0x8D */ { 0,0,0,0 }, /* 0x8E */ { 0,0,0,0 }, /* 0x8F */ { 0,0,0,0 },
+
+    /* 0x90 */ { 0,0,0,0 }, /* 0x91 */ { 0,0,0,0 }, /* 0x92 */ { 0,0,0,0 }, /* 0x93 */ { 0,0,0,0 },
+    /* 0x94 */ { 0,0,0,0 }, /* 0x95 */ { 0,0,0,0 }, /* 0x96 */ { 0,0,0,0 }, /* 0x97 */ { 0,0,0,0 },
+    /* 0x98 */ { 0,0,0,0 }, /* 0x99 */ { 0,0,0,0 }, /* 0x9A */ { 0,0,0,0 }, /* 0x9B */ { 0,0,0,0 },
+    /* 0x9C */ { 0,0,0,0 }, /* 0x9D */ { 0,0,0,0 }, /* 0x9E */ { 0,0,0,0 }, /* 0x9F */ { 0,0,0,0 },
+
+    /* 0xA0 */ { 0,0,0,0 }, /* 0xA1 */ { 0,0,0,0 }, /* 0xA2 */ { 0,0,0,0 }, /* 0xA3 */ { 0,0,0,0 },
+    /* 0xA4 */ { 0,0,0,0 }, /* 0xA5 */ { 0,0,0,0 }, /* 0xA6 */ { 0,0,0,0 }, /* 0xA7 */ { 0,0,0,0 },
+    /* 0xA8 */ { 0,0,0,0 }, /* 0xA9 */ { 0,0,0,0 }, /* 0xAA */ { 0,0,0,0 }, /* 0xAB */ { 0,0,0,0 },
+    /* 0xAC */ { 0,0,0,0 }, /* 0xAD */ { 0,0,0,0 }, /* 0xAE */ { 0,0,0,0 }, /* 0xAF */ { 0,0,0,0 },
+
+    /* 0xB0 */ { 0,0,0,0 }, /* 0xB1 */ { 0,0,0,0 }, /* 0xB2 */ { 0,0,0,0 }, /* 0xB3 */ { 0,0,0,0 },
+    /* 0xB4 */ { 0,0,0,0 }, /* 0xB5 */ { 0,0,0,0 }, /* 0xB6 */ { 0,0,0,0 }, /* 0xB7 */ { 0,0,0,0 },
+    /* 0xB8 */ { 0,0,0,0 }, /* 0xB9 */ { 0,0,0,0 }, /* 0xBA */ { 0,0,0,0 }, /* 0xBB */ { 0,0,0,0 },
+    /* 0xBC */ { 0,0,0,0 }, /* 0xBD */ { 0,0,0,0 }, /* 0xBE */ { 0,0,0,0 }, /* 0xBF */ { 0,0,0,0 },
+
+    /* 0xC0 */ { 0,0,0,0 },
+    /* 0xC1 */ { 0,0,0,0 },
+    /* 0xC2 */ { "cmpps %GX,%EX,%Ib", "cmppd %GX,%EX,%Ib", "cmpsd %GX,%EX,%Ib", "cmpss %GX,%EX,%Ib" }, // FIXME: The immediate byte specifies a comparison operator
+    /* 0xC3 */ { 0,0,0,0 },
+    /* 0xC4 */ { "pinsrw %GM,%Ed,%Ib", "pinsrw %GX,%Ed,%Ib", 0, 0 },
+    /* 0xC5 */ { "pextrw %Ed,%GM,%Ib", "pextrw %Ed,%GX,%Ib", 0, 0 },
+    /* 0xC6 */ { "shufps %GX,%EX,%Ib", "shufpd %GX,%EX,%Ib", 0, 0 },
+    /* 0xC7 */ { 0,0,0,0 },
+    /* 0xC8 */ { 0,0,0,0 }, /* 0xC9 */ { 0,0,0,0 }, /* 0xCA */ { 0,0,0,0 }, /* 0xCB */ { 0,0,0,0 },
+    /* 0xCC */ { 0,0,0,0 }, /* 0xCD */ { 0,0,0,0 }, /* 0xCE */ { 0,0,0,0 }, /* 0xCF */ { 0,0,0,0 },
+
+    /* 0xD0 */ { 0,0,0,0 },
+    /* 0xD1 */ { 0,0,0,0 },
+    /* 0xD2 */ { 0,0,0,0 },
+    /* 0xD3 */ { 0,0,0,0 },
+    /* 0xD4 */ { 0,0,0,0 },
+    /* 0xD5 */ { 0,0,0,0 },
+    /* 0xD6 */ { 0,0,0,0 },
+    /* 0xD7 */ { "pmovmskb %Gd,%EM", "pmovmskb %Gd,%EX", 0, 0 },
+    /* 0xD8 */ { 0,0,0,0 },
+    /* 0xD9 */ { 0,0,0,0 },
+    /* 0xDA */ { "pminub %GM,%EM", "pminub %GX,%EX", 0, 0 },
+    /* 0xDB */ { "pand %GM,%EM", "pand %GX,%EX", 0, 0 },
+    /* 0xDC */ { 0,0,0,0 },
+    /* 0xDD */ { 0,0,0,0 },
+    /* 0xDE */ { "pmaxub %GM,%EM", "pmaxub %GX,%EX", 0, 0 },
+    /* 0xDF */ { 0,0,0,0 },
+
+    /* 0xE0 */ { "pavgb %GM,%EM", "pavgb %GX,%EX", 0, 0 },
+    /* 0xE1 */ { 0,0,0,0 },
+    /* 0xE2 */ { 0,0,0,0 },
+    /* 0xE3 */ { "pavgw %GM,%EM", "pavgw %GX,%EX", 0, 0 },
+    /* 0xE4 */ { "pmulhuw %GM,%EM", "pmulhuw %GX,%EX", 0, 0 },
+    /* 0xE5 */ { "pmulhw %GM,%EM", "pmulhw %GX,%EX", 0, 0 },
+    /* 0xE6 */ { 0,0,0,0 },
+    /* 0xE7 */ { "movntq %EM,%GM", "movntdq %EX,%GX", 0, 0 },
+    /* 0xE8 */ { "psubsb %GM,%EM", "psubsb %GX,%EX", 0, 0 },
+    /* 0xE9 */ { "psubsw %GM,%EM", "psubsw %GX,%EX", 0, 0 },
+    /* 0xEA */ { "pminsw %GM,%EM", "pminsw %GX,%EX", 0, 0 },
+    /* 0xEB */ { "por %GM,%EM", "por %GX,%EX", 0, 0 },
+    /* 0xEC */ { "paddsb %GM,%EM", "paddsb %GX,%EX", 0, 0 },
+    /* 0xED */ { "paddsw %GM,%EM", "paddsw %GX,%EX", 0, 0 },
+    /* 0xEE */ { "pmaxsw %GM,%EM", "pmaxsw %GX,%EX", 0, 0 },
+    /* 0xEF */ { "pxor %GM,%EM", "pxor %GX,%EX", 0, 0 },
+
+    /* 0xF0 */ { 0,0,0,0 }, /* 0xF1 */ { 0,0,0,0 }, /* 0xF2 */ { 0,0,0,0 }, /* 0xF3 */ { 0,0,0,0 },
+    /* 0xF4 */ { 0,0,0,0 },
+    /* 0xF5 */ { 0,0,0,0 },
+    /* 0xF6 */ { "psadbw %GM,%EM", "psadbw %GX,%EX", 0, 0 },
+    /* 0xF7 */ { "maskmovq %GM,%EM", "maskmovdqu %GX,%EX", 0, 0 },
+    /* 0xF8 */ { 0,0,0,0 }, /* 0xF9 */ { 0,0,0,0 }, /* 0xFA */ { 0,0,0,0 }, /* 0xFB */ { 0,0,0,0 },
+    /* 0xFC */ { 0,0,0,0 }, /* 0xFD */ { 0,0,0,0 }, /* 0xFE */ { 0,0,0,0 }, /* 0xFF */ { 0,0,0,0 }
+  }
+};
+
 static char const *groups[][8] = {   /* group 0 is group 3 for %Ev set */
 /* 0 */
   { "add",            "or",              "adc",            "sbb",
@@ -385,8 +532,13 @@ static char const *groups[][8] = {   /* group 0 is group 3 for %Ev set */
     "psrad %EM,%Ib",  0,                 "pslld %EM,%Ib",  0               },
 /* ; (NTS: this is '0'+11 in ASCII) */
   { 0,                0,                 "psrlw %EM,%Ib",  0,
-    "psraw %EM,%Ib",  0,                 "psllw %EM,%Ib",  0               }
-
+    "psraw %EM,%Ib",  0,                 "psllw %EM,%Ib",  0               },
+/* < (NTS: this is '0'+12 in ASCII) */
+  { "fxsave %EM",     "fxrstor %EM",     "ldmxcsr %EM",    "stmxcsr %EM",
+    0,                0,                 0,                "sfence"        },
+/* = (NTS: this is '0'+13 in ASCII) */
+  { "prefetchnta %EM","prefetch0 %EM",   "prefetch1 %EM",  "prefetch2 %EM",
+    0,                0,                 0,                0               }
 };
 
 /* zero here means invalid.  If first entry starts with '*', use st(i) */
@@ -540,6 +692,21 @@ static void uputchar(char c)
   *ubufp = 0;
 }
 
+static void ua_backspace_repe(void) {
+  char *s = ubufp - 1;
+
+  while (s >= ubufs && (*s == ' ')) s--;
+  while (s >= ubufs && (*s != ' ')) s--;
+  s++;
+
+  if (s < ubufs) return;
+
+  if (!strncmp(s,"repe ",5) || !strncmp(s,"repne ",6)) {
+    /* set write pointer here, to overwrite it */
+    *s = 0; ubufp = s;
+  }
+}
+
 /*------------------------------------------------------------------------*/
 
 static int bytes(char c)
@@ -706,6 +873,10 @@ static void reg_name(int regnum, char size)
     uprintf("mm%d", regnum);
     return;
   }
+  if (size == 'X') { /* SSE register */
+    uprintf("xmm%d", regnum);
+    return;
+  }
   if ((((size == 'c') || (size == 'v')) && (opsize == 32)) || (size == 'd'))
     uputchar('e');
   if ((size=='q' || size == 'b' || size=='c') && !wordop) {
@@ -965,7 +1136,7 @@ static void percent(char type, char subtype)
        break;
 
   case 'S':                            /* reg(r/m) picks segment reg */
-       uputchar("ecsdfg"[REG(modrm())]);
+       uputchar("ecsdfg??"[REG(modrm())]);
        uputchar('s');
        must_do_size = 0;
        break;
@@ -991,10 +1162,17 @@ static void percent(char type, char subtype)
 
   case '2':                            /* old [pop cs]! now indexes */
        c = getbyte();
+       last_c = c;
        wordop = c & 1;
        ua_str(second[c]);              /* instructions in 386/486   */
        break;
 
+  case 'x':
+       /* problem: for SSE opcodes with REPE/REPNE mandatory prefix this code will have already written repne/repe so that needs to be wiped out of the buffer */
+       if (last_prefix == MP_F2 || last_prefix == MP_F3) ua_backspace_repe();
+       ua_str(mpgroups[subtype-'0'][last_c][last_prefix]);
+       break;
+
   case 'g':                            /* modrm group `subtype' (0--7) */
        ua_str(groups[subtype-'0'][REG(modrm())]);
        break;
@@ -1035,6 +1213,11 @@ static void percent(char type, char subtype)
        break;
 
   case 'p':                    /* prefix byte */
+       switch (last_c) {
+         case 0xF2: last_prefix = MP_F2; break;
+         case 0xF3: last_prefix = MP_F3; break;
+         default:   last_prefix = MP_NONE; break;
+       };
        switch (subtype)  {
        case 'c':
        case 'd':
@@ -1044,7 +1227,8 @@ static void percent(char type, char subtype)
        case 's':
             prefix = subtype;
             c = getbyte();
-            wordop = c & 1;
+	    last_c = c;
+	    wordop = c & 1;
             ua_str((*opmap1)[c]);
             break;
        case ':':
@@ -1053,6 +1237,7 @@ static void percent(char type, char subtype)
             break;
        case ' ':
             c = getbyte();
+	    last_c = c;
             wordop = c & 1;
             ua_str((*opmap1)[c]);
             break;
@@ -1060,10 +1245,15 @@ static void percent(char type, char subtype)
        break;
 
   case 's':                           /* size override */
+       switch (last_c) {
+         case 0x66: last_prefix = MP_66; break;
+         default:   last_prefix = MP_NONE; break;
+       };
        switch (subtype) {
        case 'a':
             addrsize = 48 - addrsize;
             c = getbyte();
+	    last_c = c;
             wordop = c & 1;
             ua_str((*opmap1)[c]);
 /*            ua_str(opmap1[getbyte()]); */
@@ -1071,7 +1261,8 @@ static void percent(char type, char subtype)
        case 'o':
             opsize = 48 - opsize;
             c = getbyte();
-            wordop = c & 1;
+	    last_c = c;
+	    wordop = c & 1;
             ua_str((*opmap1)[c]);
 /*            ua_str(opmap1[getbyte()]); */
             break;
@@ -1136,9 +1327,11 @@ Bitu DasmI386(char* buffer, PhysPt pc, uint32_t cur_ip, bool bit32)
 	if (bit32) opsize = addrsize = 32;
 	else opsize = addrsize = 16;
 	c = getbyte();
+	last_c = c;
 	wordop = c & 1;
 	must_do_size = 1;
 	invalid_opcode = 0;
+	last_prefix = MP_NONE;
 	opmap1=&op386map1;
 	ua_str(op386map1[c]);
 
