@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2020  The DOSBox Team
+ *  Copyright (C) 2002-2021  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,10 +16,11 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-
+#include <assert.h>
 #include <math.h>
 #include "dosbox.h"
 #include "inout.h"
+#include "logging.h"
 #include "pic.h"
 #include "cpu.h"
 #include "mem.h"
@@ -219,7 +220,6 @@ struct PIT_Block {
                 if (new_mode) return false;
                 if (res.cycle != 0u/*index > delay*/) return true;
                 else return false;
-                break;
             case 2:
                 if (new_mode) return true;
                 return res.counter != 0;
@@ -328,6 +328,10 @@ static void PIT0_Event(Bitu /*val*/) {
 	}
 }
 
+uint32_t PIT0_GetAssignedCounter(void) {
+    return (uint32_t)pit[0].cntr;
+}
+
 static bool counter_output(Bitu counter) {
 	PIT_Block *p = &pit[counter];
     p->track_time(PIC_FullIndex());
@@ -976,7 +980,7 @@ void TIMER_OnPowerOn(Section*) {
                 speaker_clock_lock_on = false; // PC-9821 behavior
         }
 
-        /* PC-98 has two different rates: 5/10MHz base or 8MHz base. Let the user choose via dosbox.conf */
+        /* PC-98 has two different rates: 5/10MHz base or 8MHz base. Let the user choose via dosbox-x.conf */
         pc98rate = pc98_section->Get_int("pc-98 timer master frequency");
         if (pc98rate > 6) pc98rate /= 2;
         if (pc98rate == 0) pc98rate = 5; /* Pick the most likely to work with DOS games (FIXME: This is a GUESS!! Is this correct?) */
