@@ -128,6 +128,8 @@ Bitu read_p3c0(Bitu /*port*/,Bitu /*iolen*/) {
 	return retval;
 }
  
+bool J3_IsCga4Dcga();
+
 void write_p3c0(Bitu /*port*/,Bitu val,Bitu iolen) {
 	if (!vga.internal.attrindex) {
 		attr(index)=val & 0x1F;
@@ -159,6 +161,11 @@ void write_p3c0(Bitu /*port*/,Bitu val,Bitu iolen) {
 		case 0x04:		case 0x05:		case 0x06:		case 0x07:
 		case 0x08:		case 0x09:		case 0x0a:		case 0x0b:
 		case 0x0c:		case 0x0d:		case 0x0e:		case 0x0f:
+			if(J3_IsCga4Dcga()) {
+				if(attr(index) == 0 && val != 0) {
+					val = 0;
+				}
+			}
 			if (attr(disabled) & 0x1) {
                 VGA_ATTR_SetPalette(attr(index),(uint8_t)val);
 
