@@ -34,6 +34,7 @@
 #endif
 
 extern int bootdrive;
+extern bool int13_disk_change_detect_enable;
 extern bool int13_extensions_enable, bootguest, bootvm, use_quick_reboot;
 
 diskGeo DiskGeometryList[] = {
@@ -199,8 +200,8 @@ bool getSwapRequest(void) {
     return sreq;
 }
 
-void swapInDrive(int drive) {
-    DriveManager::CycleDisks(drive, true);
+void swapInDrive(int drive, int position=0) {
+    if (!dos_kernel_disabled) DriveManager::CycleDisks(drive, true, position);
     /* Hack/feature: rescan all disks as well */
     LOG_MSG("Diskcaching reset for drive %c.", drive+'A');
     if (Drives[drive] != NULL) {
@@ -208,7 +209,8 @@ void swapInDrive(int drive) {
         Drives[drive]->MediaChange();
     }
     if (drive>1||swapInDisksSpecificDrive!=drive) return;
-    swapPosition++;
+    if (position<1) swapPosition++;
+    else swapPosition=position-1;
     if(diskSwap[swapPosition] == NULL) swapPosition = 0;
     swapInDisks(drive);
     swapping_requested = true;
@@ -217,7 +219,7 @@ void swapInDrive(int drive) {
 void swapInNextDisk(bool pressed) {
     if (!pressed)
         return;
-    DriveManager::CycleAllDisks();
+    if (!dos_kernel_disabled) DriveManager::CycleAllDisks();
     /* Hack/feature: rescan all disks as well */
     LOG_MSG("Diskcaching reset for floppy drives.");
     for(Bitu i=0;i<2;i++) { /* Swap A: and B: where DOSBox mainline would run through ALL drive letters */
@@ -236,7 +238,7 @@ void swapInNextDisk(bool pressed) {
 void swapInNextCD(bool pressed) {
     if (!pressed)
         return;
-    DriveManager::CycleAllCDs();
+    if (!dos_kernel_disabled) DriveManager::CycleAllCDs();
     /* Hack/feature: rescan all disks as well */
     LOG_MSG("Diskcaching reset for normal mounted drives.");
     for(Bitu i=2;i<DOS_DRIVES;i++) { /* Swap C: D: .... Z: if it is a CD/DVD drive */
@@ -256,6 +258,11 @@ uint8_t imageDisk::Read_Sector(uint32_t head,uint32_t cylinder,uint32_t sector,v
     if (req_sector_size != sector_size)
         return 0x05;
 
+    if (sector == 0) {
+        LOG_MSG("Attempted to read invalid sector 0.");
+        return 0x05;
+    }
+
     sectnum = ( (cylinder * heads + head) * sectors ) + sector - 1L;
 
     return Read_AbsoluteSector(sectnum, data);
@@ -679,7 +686,8 @@ static uint8_t GetDosDriveNumber(uint8_t biosNum) {
 
 static bool driveInactive(uint8_t driveNum) {
     if(driveNum>=(2 + MAX_HDD_IMAGES)) {
-        LOG(LOG_BIOS,LOG_ERROR)("Disk %d non-existant", (int)driveNum);
+        int driveCalledFor = reg_dl & 0x7f;
+        LOG(LOG_BIOS,LOG_ERROR)("Disk %d non-existent", driveCalledFor);
         last_status = 0x01;
         CALLBACK_SCF(true);
         return true;
@@ -727,7 +735,7 @@ void IDE_ResetDiskByBIOS(unsigned char disk);
 void IDE_EmuINT13DiskReadByBIOS(unsigned char disk,unsigned int cyl,unsigned int head,unsigned sect);
 void IDE_EmuINT13DiskReadByBIOS_LBA(unsigned char disk,uint64_t lba);
 
-void diskio_delay(Bits value/*bytes*/);
+void diskio_delay(Bits value/*bytes*/, int type = -1);
 
 static Bitu INT13_DiskHandler(void) {
     uint16_t segat, bufptr;
@@ -852,7 +860,10 @@ static Bitu INT13_DiskHandler(void) {
         for(i=0;i<reg_al;i++) {
             last_status = imageDiskList[drivenum]->Read_Sector((uint32_t)reg_dh, (uint32_t)(reg_ch | ((reg_cl & 0xc0)<< 2)), (uint32_t)((reg_cl & 63)+i), sectbuf);
 
-            diskio_delay(512);
+            if (drivenum < 2)
+                diskio_delay(512, 0); // Floppy
+            else
+                diskio_delay(512);
 
             /* IDE emulation: simulate change of IDE state that would occur on a real machine after INT 13h */
             IDE_EmuINT13DiskReadByBIOS(reg_dl, (uint32_t)(reg_ch | ((reg_cl & 0xc0)<< 2)), (uint32_t)reg_dh, (uint32_t)((reg_cl & 63)+i));
@@ -901,7 +912,10 @@ static Bitu INT13_DiskHandler(void) {
                 bufptr++;
             }
 
-            diskio_delay(512);
+            if(drivenum < 2)
+                diskio_delay(512, 0); // Floppy
+            else
+                diskio_delay(512);
 
             last_status = imageDiskList[drivenum]->Write_Sector((uint32_t)reg_dh, (uint32_t)(reg_ch | ((reg_cl & 0xc0) << 2)), (uint32_t)((reg_cl & 63) + i), &sectbuf[0]);
             if(last_status != 0x00) {
@@ -1037,7 +1051,7 @@ static Bitu INT13_DiskHandler(void) {
             uint64_t largesize = tmpheads*tmpcyl*tmpsect*tmpsize;
             largesize/=512;
             uint32_t ts = static_cast<uint32_t>(largesize);
-            reg_ah = (drivenum <2)?1:3; //With 2 for floppy MSDOS starts calling int 13 ah 16
+            reg_ah = (drivenum < 2)?(int13_disk_change_detect_enable?2:1):3; //With 2 for floppy MSDOS starts calling int 13 ah 16
             if(reg_ah == 3) {
                 reg_cx = static_cast<uint16_t>(ts >>16);
                 reg_dx = static_cast<uint16_t>(ts & 0xffff);
@@ -1047,7 +1061,7 @@ static Bitu INT13_DiskHandler(void) {
             if (drivenum <DOS_DRIVES && (Drives[drivenum] != 0 || drivenum <2)) {
                 if (drivenum <2) {
                     //TODO use actual size (using 1.44 for now).
-                    reg_ah = 0x1; // type
+                    reg_ah = (int13_disk_change_detect_enable?2:1); // type
 //                  reg_cx = 0;
 //                  reg_dx = 2880; //Only set size for harddrives.
                 } else {
@@ -1064,6 +1078,38 @@ static Bitu INT13_DiskHandler(void) {
             }
         }
         break;
+    case 0x16: /* Detect disk change (apparently added to XT BIOSes in 1986 according to RBIL) */
+	if (int13_disk_change_detect_enable) {
+		LOG(LOG_BIOS,LOG_WARN)("INT 13: Detect disk change");
+		if(driveInactive(drivenum)) {
+			last_status = 0x80;
+			reg_ah = last_status;
+			CALLBACK_SCF(true);
+		}
+		else if (drivenum < 2) {
+			if (imageDiskChange[drivenum]) {
+				imageDiskChange[drivenum] = false;
+				last_status = 0x06; // change line active
+				reg_ah = last_status;
+				CALLBACK_SCF(true);
+			}
+			else {
+				last_status = 0x00; // no change
+				reg_ah = last_status;
+				CALLBACK_SCF(false);
+			}
+		}
+		else {
+			last_status = 0x06; // not supported (because it's a hard drive)
+			reg_ah = last_status;
+			CALLBACK_SCF(true);
+		}
+	}
+	else {
+		reg_ah=0xff; // not supported
+		CALLBACK_SCF(true);
+	}
+	break;
     case 0x17: /* Set disk type for format */
         /* Pirates! needs this to load */
         killRead = true;
@@ -1110,7 +1156,10 @@ static Bitu INT13_DiskHandler(void) {
         for(i=0;i<dap.num;i++) {
             last_status = imageDiskList[drivenum]->Read_AbsoluteSector(dap.sector+i, sectbuf);
 
-            diskio_delay(512);
+            if(drivenum < 2)
+                diskio_delay(512, 0); // Floppy
+            else
+                diskio_delay(512);
 
             IDE_EmuINT13DiskReadByBIOS_LBA(reg_dl,dap.sector+i);
 
@@ -1145,7 +1194,10 @@ static Bitu INT13_DiskHandler(void) {
                 bufptr++;
             }
 
-            diskio_delay(512);
+            if(drivenum < 2)
+                diskio_delay(512, 0); // Floppy
+            else
+                diskio_delay(512);
 
             last_status = imageDiskList[drivenum]->Write_AbsoluteSector(dap.sector+i, &sectbuf[0]);
             if(last_status != 0x00) {
