@@ -21,7 +21,9 @@
  * 02110-1301, USA
  */
 
-#if !C_FLUIDSYNTH && defined(WIN32)
+#define FLUIDINC
+#include "config.h"
+#if !C_FLUIDSYNTH && defined(WIN32) && !defined(C_HX_DOS)
 #include "fluid_defsfont.h"
 /* Todo: Get rid of that 'include' */
 #include "fluid_sys.h"
